#########
# Author:        gq1
# Maintainer:    $Author: gq1 $
# Created:       2008-10-15
# Last Modified: $Date: 2008-10-30 12:47:27 +0000 (Thu, 30 Oct 2008) $
# Id:            $Id: clean_repository.pm 3347 2008-10-30 12:47:27Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-3.0/lib/npg_qc/api/clean_repository.pm $
#
package npg_qc::api::clean_repository;
use strict;
use warnings;
use Carp;
use srpipe::util;
use English qw{-no_match_vars};
use base qw(npg_qc::api::base);



our $VERSION = do { my ($r) = q$LastChangedRevision: 3347 $ =~ /(\d+)/mx; $r; };

## no critic 

my %both_pair_run_complete_of    :ATTR( :get<both_pair_run_complete>, :set<both_pair_run_complete> );
my %one_pair_run_complete_of     :ATTR( :get<one_pair_run_complete>, :set<one_pair_run_complete> );

my %both_pair_run_complete_no_repos_of    :ATTR( :get<both_pair_run_complete_no_repos>, :set<both_pair_run_complete_no_repos> );
my %one_pair_run_complete_no_repos_of     :ATTR( :get<one_pair_run_complete_no_repos>, :set<one_pair_run_complete_no_repos> );

my %both_pair_run_complete_files_of    :ATTR( :get<both_pair_run_complete_files>, :set<both_pair_run_complete_files> );
my %one_pair_run_complete_files_of     :ATTR( :get<one_pair_run_complete_files>, :set<one_pair_run_complete_files> );
my %none_pair_run_complete_files_of    :ATTR( :get<none_pair_run_complete_files>, :set<none_pair_run_complete_files> );

##use critic

sub report{
  my ($self, $project_name, $file_ext_name, $web_target) = @_;

  $self->parse_runlist_xml($web_target);

  $self->retrieve_repository_file_list($project_name, $file_ext_name);

  print "\n>>The number of run pairs, with complete data in the database: "
    or croak 'Can not print';
  print scalar keys %{$self->get_both_pair_run_complete()}, "\n" or croak 'Can not print';

  print "\n>>The number of run pairs, only one run of the pair with complete data in the database: "
    or croak 'Can not print';
  print scalar keys %{$self->get_one_pair_run_complete()}, "\n" or croak 'Can not print';

  my @both_pair_run_complete_no_repos_list = keys %{$self->get_both_pair_run_complete_no_repos()};
  $self->report_file_list(\@both_pair_run_complete_no_repos_list , 'The run list, both runs of the pair with complete data in the database but no repository file');

  my @one_pair_run_complete_no_repos_list = keys %{$self->get_one_pair_run_complete_no_repos()};
  $self->report_file_list(\@one_pair_run_complete_no_repos_list, 'The run list, just one run of the pair with complete data in the database but no repository file');

  $self->report_file_list($self->get_both_pair_run_complete_files(), 'The repository file list both run of the pair with complete data in the database');
  $self->report_file_list($self->get_one_pair_run_complete_files(), 'The repository file list, just one run of the pair with complete data in the database');
  $self->report_file_list($self->get_none_pair_run_complete_files(), 'The repository file list, both of run of the pair without complete data in the database');

  return 1;
}

sub report_file_list{
  my ($self, $file_list, $message) = @_;
  
  print "\n>>$message: " or croak 'Can not print';
  
  print scalar @{$file_list}, "\n" or croak 'Can not print';

  foreach my $file (sort @{$file_list}){
    print "$file\n" or croak 'Can not print';
  }

  return 1;
}

sub retrieve_repository_file_list {
    my ($self, $project_name, $file_ext_name) = @_;

    my $srpipe_util = srpipe::util->new();

    my $repository_list = $srpipe_util->pfind($project_name, 1);

    $self->set_both_pair_run_complete_files([]);
    $self->set_one_pair_run_complete_files([]);
    $self->set_none_pair_run_complete_files([]);

    foreach my $repository (@{$repository_list}){

      my @repository_files
        = $self->my_glob($repository, $file_ext_name);

	   foreach my $file (@repository_files ){

        my ($id_run_pair) = $file =~ /(\d+\+?\d+)\-/xms ;

        if($self->get_both_pair_run_complete()->{$id_run_pair}){

           push @{$self->get_both_pair_run_complete_files}, $file;
           delete $self->get_both_pair_run_complete_no_repos()->{$id_run_pair};
        }elsif($self->get_one_pair_run_complete()->{$id_run_pair}){

           push @{$self->get_one_pair_run_complete_files()}, $file;
           delete $self->get_one_pair_run_complete_no_repos()->{$id_run_pair};
        }else{

          push @{$self->get_none_pair_run_complete_files()}, $file;
        }
      }
    }

    #print @{$self->get_both_pair_run_complete_files()}, "\n";    
    #print @{$self->get_one_pair_run_complete_files()}, "\n";    
    #print @{$self->get_none_pair_run_complete_files()}, "\n";
    #print keys %{$self->get_both_pair_run_complete}, "\n";
    #print keys %{$self->get_one_pair_run_complete}, "\n"; 
   
    return 1;
}

sub my_glob {
  my ($self, $root, $dir) = @_;

  if(!$dir) {
    $dir  = $root;
    $root = q[/];
    $dir  =~ s{^/}{}mx;
  }
  my ($head, $tail) = split m{/}mx, $dir, 2;

  $head =~ s/\*/.*/mxg;
  $tail ||= q[];

  opendir my $dh, "$root";
  my @heads = map  { m{^([a-z\d_,/\+\.\-\ ]+)$}mix }
              grep { /$head/mx }
	      grep { $_ !~ /^\./mx }
              readdir $dh;
  closedir $dh;

  if($root eq q[/]) {
    $root = q[];
  }

  if($tail) {
    return map { $self->my_glob("$root/$_", $tail); } @heads;
  }

  return map { "$root/$_" } @heads;
}

sub parse_runlist_xml{
    my ($self, $web_target) = @_;

    my $util = $self->util();

    my $runlist_xml;

    my $both_pair_run_complete;
    my $one_pair_run_complete;

    $self->set_both_pair_run_complete([]);
    $self->set_one_pair_run_complete([]);

    eval{
      $runlist_xml = $self->web_get_xml($web_target);
      1;
    } or do {
      croak "no runlist xml: $EVAL_ERROR";
    };

    if($runlist_xml){

      my $parser = $self->util->parser();
      my $runlist_element = $parser->parse_string($runlist_xml)->getElementsByTagName('run');

      foreach my $run_pair (@{$runlist_element}){

        my $id_run = $run_pair->getAttribute('id_run');
        my $run_complete = $run_pair->getAttribute('run_complete');
        my $id_run_pair = $run_pair->getAttribute('id_run_pair');
        my $pair_complete = $run_pair->getAttribute('pair_complete');
        #print "$id_run, $run_complete, $id_run_pair, $pair_complete: ";

        if($id_run && $id_run_pair){

          if($run_complete && $pair_complete){

            $both_pair_run_complete->{qq{$id_run+$id_run_pair}} = 1;
            #print "both finish\n";
          }elsif(!$run_complete || !$pair_complete){

            $one_pair_run_complete->{qq{$id_run+$id_run_pair}} = 1;
            #print "only one finish\n";
          }
        }else{

          $both_pair_run_complete->{$id_run} = 1;
          #print "only one run\n";
        }
      }
      $self->set_both_pair_run_complete($both_pair_run_complete);
      $self->set_one_pair_run_complete($one_pair_run_complete);
      
      my %both_pair_run_complete_no_repos = %{$both_pair_run_complete};
      my %one_pair_run_complete_no_repos = %{$one_pair_run_complete};

      $self->set_both_pair_run_complete_no_repos(\%both_pair_run_complete_no_repos);
      $self->set_one_pair_run_complete_no_repos(\%one_pair_run_complete_no_repos);
      #print keys %{$self->get_both_pair_run_complete}, "\n";
      #print keys %{$self->get_one_pair_run_complete}, "\n";

    }
    return 1;
}

sub web_get_xml {
    my ($self, $web_target) = @_;

    my $util = $self->util();

    my $response = $util->get($util->base_uri().$web_target);

    my $xml;

    if ($response->is_success()) {

      $xml = $response->content();
    } else {

      croak 'failed to successfully get xml - this could be from a timeout issue';
    }

    return $xml;
}

1;
__END__

=head1 NAME

npg_qc::api::clean_repository

=head1 VERSION

$LastChangedRevision: 3347 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 web_get_xml - given an web address, get the response content

=head2 parse_runlist_xml - parsing the runlist xml file

=head2 my_glob - get a list of matching files under one directory

=head2 retrieve_repository_file_list - check the file list in the repository again the runlist in xml

=head2 report - make the final report

=head2 report_file_list - make one report

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Carp
English
npg_qc::api::base
srpipe::util

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Guoying Qi

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
