#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-09-29
# Last Modified: $Date: 2008-10-30 12:47:27 +0000 (Thu, 30 Oct 2008) $
# Id:            $Id: summary.pm 3347 2008-10-30 12:47:27Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-3.0/lib/npg_qc/api/summary.pm $
#
package npg_qc::api::summary;
use strict;
use warnings;
use Class::Std;
use Carp;
use base qw(npg_qc::api::base);
use Readonly;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3347 $ =~ /(\d+)/mx; $r; };

{
  ## no critic
  my %id_run_of        :ATTR( :get<id_run>,        :set<id_run>        );
  my %end_of           :ATTR( :get<end>,           :set<end>           );
  my %paired_of        :ATTR( :get<paired>,        :set<paired>        );
  my %summaries_of     :ATTR( :get<summaries>,     :set<summaries>     );
  my %complete_runs_of :ATTR( :get<complete_runs>, :set<complete_runs> );
  ## use critic

  sub fields {
    return qw(
              id_run
              end
              paired
            );
  }

  sub id_run {
    my ($self, $id_run) = @_;
    if ($id_run) {
      $self->set_id_run($id_run);
    }
    return $self->get_id_run();
  }

  sub end {
    my ($self, $end) = @_;
    if ($end) {
      $self->set_end($end);
    }
    return $self->get_end();
  }

  sub paired {
    my ($self, $paired) = @_;
    if ($paired) {
      $self->set_paired($paired);
    }
    return $self->get_paired();
  }

  sub complete_runs {
    my ($self) = @_;

    if (!$self->get_complete_runs()) {
      my $util = $self->util();
      my $pkg        = ref $self;
      my ($obj_type) = $pkg =~ /([^:]+)$/mx;
      my $obj_pk     = $self->primary_key();
      my $obj_uri    = sprintf '%s/%s', $util->base_uri(), $obj_type;
      my $response   = $util->get($obj_uri);

      if(!$response->is_success()) {
        croak $response->status_line();
      }

      $self->set_summaries($util->parser->parse_string($response->content()));
      my $complete_run_summaries = $self->get_summaries()->getElementsByTagName('complete_run_summaries')->[0];

      my $store = [map { $self->new_from_xml($pkg, $_) } $complete_run_summaries->getElementsByTagName('run')];
      $self->set_complete_runs($store);
    }

    return $self->get_complete_runs();
  }

  sub runs_with_fully_complete_pairs {
    my ($self) = @_;
    my $paired_runs;
    my $complete_runs;
    my $pair_one;
    my $pair_two;
    foreach my $run (@{$self->complete_runs()}) {
      if (!$run->paired()) {
        push @{$complete_runs}, $run;
      } else {
        if ($run->end == 1 && !$pair_one->{$run->id_run()}) {
          push @{$paired_runs->{$run->id_run()}}, $run;
	  $pair_one->{$run->id_run()}++;
	} elsif ($run->end == 2 && !$pair_two->{$run->id_run()}) {
          push @{$paired_runs->{$run->id_run()}}, $run;
	  $pair_two->{$run->id_run()}++;
	} else {
	}
      }
    }
    foreach my $run (sort keys %{$paired_runs}) {
      if (scalar@{$paired_runs->{$run}} == 2) {
        push @{$complete_runs}, $paired_runs->{$run}->[0];
      }
    }
    return $complete_runs;
  }

}
1;
__END__

=head1 NAME

npg_qc::api::summary

=head1 VERSION

$LastChangedRevision: 3347 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 new - Constructor

  my $oSummary = npg_qc::api::summary->new({'util' => $oUtil,});

=head2 fields - returns an array of the fields in the run_config table

  my $aFields = $oSummary->fields();

=head2 id_run - accessor for id_run. takes an id as an optional parameter (which it stores) and returns the id_run

  my $id_run = $oSummary->id_run($id_run);

=head2 end - accessor for end. takes an end as an optional parameter (which it stores) and returns the end

  my $end = $oSummary->end($end);

=head2 util - util instance for this resource set

  Returns a cached object if one exists, or creates and caches a new one

  my $oUtil = $oDerived->util();

=head2 complete_runs - fetches all complete runs from the npg_qc application, and turns into array of npg_qc::api::summary objects

  my $aCompleteRuns = $oSummary->complete_runs();

=head2 new_from_xml - create and populate an npg::api::* from a XML::LibXML::Element

  my $oObj = $oDerived->new_from_xml('npg::api::<package>', $oXMLElement);

=head2 get - overridden get accessor from Class::Accessor

Fetches XML over HTTP based on self->util->base_uri().

=head2 list - default handling for reading lists of objects

  my $oDOM = $oDerived->list();

=head2 read - default handling for reading individual objects

  my $oDOM = $oDerived->read();

=head2 create - default handling object creation

  my $oDOM = $oDerived->create();

  This creates an HTTP POST request with a multipart/form-data flavour
  and pushes object member data, for a new entity, up to the
  create_xml aspects of the service.

=head2 update - default handling object update

  my $oDOM = $oDerived->update();

  This creates an HTTP POST request with a multipart/form-data flavour
  and pushes object member data, for an existing entity ,up to the
  update_xml aspects of the service.

=head2 retry - method retries connecting to npg_qc before croaking with an error=head1 DIAGNOSTICS

=head2 paired
=head2 runs_with_fully_complete_pairs


=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
Carp
npg_qc::api::base
Readonly

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
