#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-11
# Last Modified: $Date: 2008-10-30 12:47:27 +0000 (Thu, 30 Oct 2008) $
# Id:            $Id: util.pm 3347 2008-10-30 12:47:27Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-3.0/lib/npg_qc/api/util.pm $
#
package npg_qc::api::util;
use base qw(Class::Accessor);
use strict;
use warnings;
use Class::Std;
use HTTP::Request::Common;
use LWP::UserAgent;
use XML::LibXML;
use npg::api::util;
use Carp;
use Readonly;

our $VERSION  = do { my ($r) = q$LastChangedRevision: 3347 $ =~ /(\d+)/mx; $r; };
our $DEBUG    = 0;

Readonly our $TIMEOUT => 60;

{
  ## no critic
  my %base_uri_of  :ATTR( init_arg => 'base_uri',     :get<base_uri>,     :set<base_uri> );
  my %npg_uri_of   :ATTR( init_arg => 'npg_uri',      :get<npg_uri>,      :set<npg_uri> );
  my %useragent_of :ATTR( init_arg => 'useragent',    :get<useragent>,    :set<useragent> );
  my %parser_of    :ATTR( init_arg => 'parser',       :get<parser>,       :set<parser> );
  my %npg_api_util :ATTR( init_arg => 'npg_api_util', :get<npg_api_util>, :set<npg_api_util> );
  ## use critic

  sub BUILD {
    my ($self, @args) = @_;
    $args[1]->{base_uri}     || $self->set_base_uri('http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg_qc/npg_qc/');
    $args[1]->{npg_uri}      || $self->set_npg_uri('http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/');
    $args[1]->{parser}       || $self->set_parser(XML::LibXML->new());
    $args[1]->{npg_api_util} || $self->set_npg_api_util(npg::api::util->new());
    return;
  }

  sub base_uri {
    my ($self, $base_uri) = @_;
    if ($base_uri) {
      $self->set_base_uri($base_uri);
    }
    return $self->get_base_uri();
  }

  sub npg_uri {
    my ($self, $npg_uri) = @_;
    if ($npg_uri) {
      $self->set_npg_uri($npg_uri);
    }
    return $self->get_npg_uri();
  }

  sub useragent {
    my $self = shift;

    if(!$self->get_useragent()) {
      my $ua = LWP::UserAgent->new();
      $ua->agent("npg_qc::api::util $VERSION");
      $ua->timeout($TIMEOUT);
      $ua->env_proxy();
      $self->set_useragent($ua);
    }
    return $self->get_useragent();
  }

  sub parser {
    my $self = shift;
    $self->get_parser() || $self->set_parser(XML::LibXML->new());
    return $self->get_parser();
  }

  sub get {
    my ($self, $uri, $args) = @_;
    my $req = GET $uri, 'Accept' => 'text/xml', @{$args||[]};
    return $self->useragent->request($req);
  }

  sub post {
    my ($self, $uri, $args) = @_;
    my $req = POST $uri, 'Accept' => 'text/xml', @{$args||[]};
    return $self->useragent->request($req);
  }

}

1;
__END__

=head1 NAME

npg_qc::api::util

=head1 VERSION

$LastChangedRevision: 3347 $

=head1 SYNOPSIS

  my $oUtil = npg_qc::api::util->new({
    useragent => $oUserAgent,
    base_uri  => $sBaseURI,
    parser    => $oXmlParser,
  });

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 BUILD - Additional functions to constructor, sets default uri and parser if not provided

=head2 base_uri - gets and sets base uri for this resource

  my $sURI = $oDerivedObject->base_uri();
  my $sNEW_URI = $oDerivedObject->base_uri($new_uri);

=head2 npg_uri - gets and sets npg uri for this resource

  my $sURI = $oUtil->npg_uri();
  my $sNEW_URI = $oUil->npg_uri($new_uri);

=head2 useragent - accessor to obtain LWP::UserAgent object

  my $oUserAgent = $oDerivedObject->useragent();

=head2 parser - accessor to obtain Xml_Parser object stored, or provide libxml2 XML::LibXML parser object by default

  my $oLibXMLParser = $oDerivedObject->parser();

=head2 get - HTTP GET with additional Accept: text/xml header

  my $oHTTPResponse = $oUtil->get($sURI, $arHTTPRequestHeaders);

=head2 post - HTTP POST with additional Accept: text/xml header

  my $oHTTPResponse = $oUtil->post($sURI, $arHTTPRequestHeaders);

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

Class::Std
Carp
LWP::UserAgent
HTTP::Request
XML::LibXML
Readonly
npg::api:util

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
