#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-10
# Last Modified: $Date: 2008-10-30 12:47:27 +0000 (Thu, 30 Oct 2008) $
# Id:            $Id: errors_by_cycle.pm 3347 2008-10-30 12:47:27Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-3.0/lib/npg_qc/model/errors_by_cycle.pm $
#

package npg_qc::model::errors_by_cycle;
use strict;
use warnings;
use base qw(npg_qc::model);
use English qw{-no_match_vars};
use Readonly;
use Carp;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3347 $ =~ /(\d+)/mx; $r; };

Readonly our $MAX_ERROR_PERCENTAGE => 20; # max error percentage set to 20
Readonly our $HEATMAP_VALUE_GT_20  => 100;
Readonly our $HEATMAP_VALUE_GT_5   => 50;
Readonly our $FIVE                 => 5;
Readonly our $FOURTH_ARRAY_INDEX   => 3;

__PACKAGE__->mk_accessors(__PACKAGE__->fields());

sub fields {
  return qw(
            id_errors_by_cycle
            id_run_tile
            cycle
            error_percentage
            blank_percentage
            rescore
          );
}

sub heatmap_with_map { ## no critic
  my ($self, $id_run, $end, $dataset, $url, $cycle) = @_;

  my $data_refs = {
    id_run    => $id_run,
    end       => $end,
    dataset   => $dataset,
    image_url => $url,
    id        => $dataset . q{:} . $id_run . q{:} . $end,
    hover_map => 1,
  };

  my $data_array = $self->heatmap_data($data_refs);

  my $heatmap_obj = npg_common::image::heatmap->new({
    data_array => $data_array,
  });

  eval {
    $heatmap_obj->plot_illumina_map($data_refs);
    $data_refs->{data} = $heatmap_obj->image_map_reference();
    foreach my $box (@{$data_refs->{data}}) {
      my $data_information = $box->[-1];
      my $params = q{id_run=} . $id_run . q{&position=} . $data_information->{position} . q{&tile=} . $data_information->{tile} . q{&end=} . $end . q{&cycle=} . $cycle;
      $box->[-1]->{url} = q{#" onClick="open_tile_viewer('/cgi-bin/prodsoft/npg_qc/npg_qc/run_tile/;read_tile_viewer?} . $params .q{');return false;};
    }
    my $image_map_object = npg_common::image_map->new();
    $self->{map} = $image_map_object->render_map($data_refs);
  } or do {
    croak 'Unable to render map: ' . $EVAL_ERROR;
  };

  return $self->{map};
}

sub run_tile {
  my $self  = shift;
  my $pkg   = 'npg_qc::model::run_tile';
  return $pkg->new({
		    'util' => $self->util(),
		    'id_run_tile' => $self->id_run_tile(),
		   });
}

sub data_for_plot {
  my ($self, $id_run, $position, $tile) = @_;

  my @rows;
  eval {

    my $cycles;
    my $dbh = $self->util->dbh();
    my $query = q{SELECT ec.cycle, ec.error_percentage
                  FROM   run_tile rt,
                         errors_by_cycle ec
                  WHERE  rt.id_run = ?
                  AND    rt.position = ?
                  AND    rt.tile = ?
                  AND    rt.end = 1
                  AND    rt.id_run_tile = ec.id_run_tile
                  AND    rescore = 1
                  GROUP BY cycle,position
                  ORDER BY position, cycle};
    my $sth = $dbh->prepare($query);
    $sth->execute($id_run, $position, $tile);

    while (my @row = $sth->fetchrow_array()) {
      if ($row[1] > $MAX_ERROR_PERCENTAGE) {
        $row[1] = $MAX_ERROR_PERCENTAGE;
      }
      push @rows, \@row;
      $cycles = $row[0];

    }

    $query = q{SELECT ec.cycle, ec.error_percentage
               FROM   run_tile rt,
                      errors_by_cycle ec
               WHERE  rt.id_run = ?
               AND    rt.position = ?
               AND    rt.tile = ?
               AND    rt.end = 2
               AND    rt.id_run_tile = ec.id_run_tile
               AND    rescore = 1
               GROUP BY cycle,position
               ORDER BY position, cycle};
    $sth = $dbh->prepare($query);
    $sth->execute($id_run, $position, $tile);

    while (my @row = $sth->fetchrow_array()) {
      if ($row[1] > $MAX_ERROR_PERCENTAGE) {
        $row[1] = $MAX_ERROR_PERCENTAGE;
      }
      $row[0] += $cycles;
      push @rows, \@row;
    }

    1;

  } or do {
    croak $EVAL_ERROR;
  };

  if (!scalar@rows) {
    return [];

  }

  my $held_row = shift @rows;
  my $cycle = $held_row->[0];
  my $return_array;

  foreach my $row (@rows) {
    if ($row->[0] != $cycle) {
      push @{$return_array}, $held_row;
      $cycle = $row->[0];
    }
    $held_row = $row;
  }
  push @{$return_array}, $held_row;

  return $return_array;
}

sub data_for_plot_including_pre_chastity {
  my ($self, $id_run, $position, $tile) = @_;
  my @rows;
  my %collated;
  eval {

    my $cycles;
    my $dbh = $self->util->dbh();
    my $query = q{SELECT ec.cycle, rescore, ec.error_percentage
                  FROM   run_tile rt,
                         errors_by_cycle ec
                  WHERE  rt.id_run = ?
                  AND    rt.position = ?
                  AND    rt.tile = ?
                  AND    rt.end = 1
                  AND    rt.id_run_tile = ec.id_run_tile
                  ORDER BY rescore, cycle};
    my $sth = $dbh->prepare($query);
    $sth->execute($id_run, $position, $tile);

    while (my @row = $sth->fetchrow_array()) {
      if ($row[2] > $MAX_ERROR_PERCENTAGE) {
        $row[2] = $MAX_ERROR_PERCENTAGE;
      }
      $collated{$row[0]}->{$row[1]} = $row[2];
      $cycles = $row[0];

    }

    $query = q{SELECT ec.cycle, rescore, ec.error_percentage
               FROM   run_tile rt,
                      errors_by_cycle ec
               WHERE  rt.id_run = ?
               AND    rt.position = ?
               AND    rt.tile = ?
               AND    rt.end = 2
               AND    rt.id_run_tile = ec.id_run_tile
               ORDER BY rescore, cycle};
    $sth = $dbh->prepare($query);
    $sth->execute($id_run, $position, $tile);

    while (my @row = $sth->fetchrow_array()) {
      if ($row[2] > $MAX_ERROR_PERCENTAGE) {
        $row[2] = $MAX_ERROR_PERCENTAGE;
      }
      $row[0] += $cycles;
      $collated{$row[0]}->{$row[1]} = $row[2];
    }

    foreach my $cycle (sort { $a <=> $b } keys %collated) {
      push @rows, [$cycle, $collated{$cycle}{0}, $collated{$cycle}{1}];
    }

    1;

  } or do {
    croak $EVAL_ERROR;
  };

  if (!scalar@rows) {
    return [];

  }

  my $held_row = shift @rows;
  my $cycle = $held_row->[0];
  my $return_array;

  foreach my $row (@rows) {
    if ($row->[0] != $cycle) {
      push @{$return_array}, $held_row;
      $cycle = $row->[0];
    }
    $held_row = $row;
  }
  push @{$return_array}, $held_row;

  return $return_array;
}

sub average_perc_for_lane {
  my ($self, $id_run, $position) = @_;

  my @rows;

  eval {

    my $dbh = $self->util->dbh();
    my $query = q{SELECT ec.cycle, AVG(ec.error_percentage) AS error_percentage
                  FROM   run_tile rt,
                         errors_by_cycle ec
                  WHERE  rt.id_run = ?
                  AND    rt.position = ?
                  AND    rt.end = 1
                  AND    rt.id_run_tile = ec.id_run_tile
                  AND    rescore = 1
                  GROUP BY cycle,position
                  ORDER BY position, cycle};
    my $sth = $dbh->prepare($query);
    $sth->execute($id_run, $position);

    while (my @row = $sth->fetchrow_array()) {
      if ($row[1] > $MAX_ERROR_PERCENTAGE) {
        $row[1] = $MAX_ERROR_PERCENTAGE;
      }
      push @rows, \@row;
    }

    $query = q{SELECT ec.cycle, AVG(ec.error_percentage) AS error_percentage
               FROM   run_tile rt,
                      errors_by_cycle ec
               WHERE  rt.id_run = ?
               AND    rt.position = ?
               AND    rt.end = 2
               AND    rt.id_run_tile = ec.id_run_tile
               AND    rescore = 1
               GROUP BY cycle,position
               ORDER BY position, cycle};
    $sth = $dbh->prepare($query);
    $sth->execute($id_run, $position);

    my $cycles = $rows[-1]->[0];

    while (my @row = $sth->fetchrow_array()) {
      if ($row[1] > $MAX_ERROR_PERCENTAGE) {
        $row[1] = $MAX_ERROR_PERCENTAGE;
      }
      $row[0] += $cycles;
      push @rows, \@row;
    }

    1;

  } or do {

    croak $EVAL_ERROR;

  };

  return \@rows;
}

sub errors_by_tile {
  my ($self, $tile) = @_;

  my @errors_by_tile = sort { $a->cycle() <=> $b->cycle } @{$tile->errors_by_cycles()};

  return @errors_by_tile;
}

sub cycle_count {
  my ($self) = @_;
  
  if (!$self->{cycle_count}) {
    my $id_run = $self->{id_run};
    my $dbh = $self->util->dbh();

    my $query = qq{SELECT max(ec.cycle) FROM run_tile rt, errors_by_cycle ec WHERE rt.id_run = $id_run AND ec.id_run_tile = rt.id_run_tile};
    my $sth = $dbh->prepare($query);
    $sth->execute();

    $self->{cycle_count} = $sth->fetchrow_array();
  }

  return $self->{cycle_count};
}

sub cycles_for_run {
  my ($self) = @_;

  my $id_run = $self->{id_run};
  my $dbh = $self->util->dbh();

  my $cycle_count = $self->cycle_count();

  my $query = qq{SELECT max(end) FROM run_tile WHERE id_run = $id_run};
  my $sth = $dbh->prepare($query);
  $sth->execute();

  my $end = $sth->fetchrow_array();
  
  if ($end == 2) {
    $cycle_count *= 2;
  }

  return [1..$cycle_count];
}

sub heatmap_data_per_cycle {
  my ($self) = @_;
  my $cycle = $self->cycle();
  my $id_run = $self->{id_run};
  my $dbh = $self->util->dbh();

  my $query = qq{SELECT max(tile) FROM run_tile WHERE id_run = $id_run};
  my $sth = $dbh->prepare($query);
  $sth->execute();
  my $tile_count = $sth->fetchrow_array();

  $query = qq{SELECT max(ec.cycle) FROM run_tile rt, errors_by_cycle ec WHERE rt.id_run = $id_run AND ec.id_run_tile = rt.id_run_tile};
  $sth = $dbh->prepare($query);
  $sth->execute();
  my $cycle_count = $sth->fetchrow_array();

  my $end = 1;

  if ($cycle > $cycle_count) {
    $end = 2;
    $cycle = $cycle - $cycle_count;
  }

  $query = q{SELECT DISTINCT rt.position, rt.tile, ec.cycle, ec.error_percentage
             FROM   run_tile rt,
                    errors_by_cycle ec
             WHERE  rt.id_run = ?
             AND    rt.end = ?
             AND    rt.id_run_tile = ec.id_run_tile
             AND    ec.rescore = 1
             AND    ec.cycle = ?
             ORDER BY cycle, position, tile};

  $dbh = $self->util->dbh();
  $sth = $dbh->prepare($query);
  $sth->execute($id_run, $end, $cycle);

  my $data = [[],[],[],[],[],[],[],[]];

  while (my @row = $sth->fetchrow_array) {
    my $position = $row[0] - 1;

    my $value = $row[$FOURTH_ARRAY_INDEX] > $MAX_ERROR_PERCENTAGE ? $HEATMAP_VALUE_GT_20
              : $row[$FOURTH_ARRAY_INDEX] > $FIVE                 ? $HEATMAP_VALUE_GT_5
              :                                                     $row[$FOURTH_ARRAY_INDEX]
              ;

    push @{$data->[$position]}, $value;
  }

  foreach my $array (@{$data}) {

    if (scalar @{$array} == 0) {

      my $temp_count = $tile_count - 1;
      for my $i (0..$temp_count) {
        push @{$array}, 0;
      }

    }

  }

  return $data;
}

1;
__END__
=head1 NAME

npg_qc::model::errors_by_cycle

=head1 VERSION

$LastChangedRevision: 3347 $

=head1 SYNOPSIS

  my $oErrorsByCycle = npg_qc::model::errors_by_cycle->new({util => $util});

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 fields - return array of fields, first of which is the primary key

  my $aFields = $oErrorsByCycle->fields();

=head2 run_tile - returns run_tile object that this object belongs to

  my $oRunTile = $oErrorsByCycle->run_tile();

=head2 data_for_plot - generates the data to plot errors_by_cycle for a lane in order to be plotted

  my $aDataForPlot = $oErrorsByCycle->data_for_plot($id_run, $position, $tile);

=head2 errors_by_tile - fetches the errors by cycle for a given tile

  my @ErrorsByTile = $oErrorsByCycle->errors_by_tile($oRunTile);

=head2 run_tiles - returns an arrayref of all the run_tiles for a given run (in $self->{id_run})

  my $aRunTiles = $oErrorsByCycle->run_tiles();

=head2 run_tiles_uniq_names - returns an arrayref of unique tile names in the format <id_run>_<lane>_<tile_no>

  my $aRunTilesUnigNames = $oErrorsByCycle->run_tiles_uniq_names();

=head2 average_perc_for_lane - returns the data to plot average percentage errors by cycle for a lane

  my $aAveragePercForLane = $oErrorsByCycle->average_perc_for_lane($id_run, $position);

=head2 lanes - returns an array of all lanes for a run (provided id_run has already been put onto object)

  my $aLanes = $oErrorsByCycle->lanes();

=head2 heatmap_data_per_cycle - obtains the error percentages per tile for a given cycle

=head2 cycle_count - returns (and caches) the cycle count for a single run of the run/runpair

  my $iCycleCount = $oErrorsByCycle->cycle_count();

=head2 cycles_for_run - returns an arrayref containing all the cycle numbers for a run (or run/runpair)

  my $aCyclesForRun = $oErrorsByCycle->cycles_for_run();

=head2 heatmap_with_map - returns some html code with a heatmap url and a hovermap for it

=head2 data_for_plot_including_pre_chastity - returns the data required to draw a plot with two lines, pre and post chastity

  my $aDataForPlotIncludingChastity = $oErrorsByCycle->data_for_plot_including_pre_chastity($id_run, $position, $tile);

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
npg_qc::model
English
Readonly

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
