#########
# Author:        gq1
# Maintainer:    $Author: gq1 $
# Created:       2008-09-29
# Last Modified: $Date: 2008-10-30 12:47:27 +0000 (Thu, 30 Oct 2008) $
# Id:            $Id: run_graph.pm 3347 2008-10-30 12:47:27Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-3.0/lib/npg_qc/model/run_graph.pm $
#

package npg_qc::model::run_graph;
use strict;
use warnings;
use base qw(npg_qc::model);
use English qw(-no_match_vars);
use Carp;
use Readonly;
use npg_qc::model::cache_query;
use npg_qc::model::instrument_statistics;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3347 $ =~ /(\d+)/mx; $r; };


Readonly our $PERCENTAGE      => 100;
Readonly our $GIGABASE        => 1_000_000;


__PACKAGE__->mk_accessors(__PACKAGE__->fields());
__PACKAGE__->has_all();

sub fields {
  return qw(
            id_run_graph
            id_run
            yield_gb
            avg_error
            avg_cluster_per_tile
            avg_cluster_per_tile_raw
            avg_cluster_per_tile_control
            avg_cluster_per_tile_raw_control
          );
}
#given id_run, find primary key if there is one
sub init {
  my $self = shift;
  
  if($self->id_run()
     &&!$self->id_run_graph()) {

    my $query = q(SELECT id_run_graph
                  FROM   run_graph
                  WHERE  id_run = ?
                 );

    my $ref   = [];

    eval {
      $ref = $self->util->dbh->selectall_arrayref($query, {}, $self->id_run());
      1;
    } or do {
      carp $EVAL_ERROR;
      return;
    };

    if(@{$ref}) {
      $self->id_run_graph($ref->[0]->[0]);
    }
  }
  return 1;
}
#given id_run, end and the cycle count of this run
#get total yield, avg error and avg cluster number per tile for this run
sub get_total_yield_error_cluster{
  my ($self, $id_run, $end, $cycle) = @_;
  
  my $query = q{   
         SELECT ROUND(SUM(temp.clusters_raw*temp.perc_pf_clusters*?/?)) AS total_pf_yield,
                ROUND(AVG(temp.perc_error_rate_pf),2) AS perc_error_rate,
                ROUND(AVG(temp.clusters_raw*temp.perc_pf_clusters/100)) AS clusters_pf,
                ROUND(AVG(temp.clusters_raw)) AS clusters_raw
         FROM  (SELECT DISTINCT position, tile, clusters_raw, perc_pf_clusters, perc_error_rate_pf
                         FROM  lane_qc lqc, run_tile rt
                         WHERE rt.id_run = ?
                         AND   rt.id_run_tile = lqc.id_run_tile
                         AND   lqc.end = ?
                         ORDER BY position, tile) temp
               };
  eval {
      my $dbh = $self->util->dbh();
     
      my $sth = $dbh->prepare($query);
      
      $sth->execute($cycle,($GIGABASE*$PERCENTAGE), $id_run, $end);

      my @row = $sth->fetchrow_array();
      if(@row){
        $self->yield_gb($row[0]);
        $self->avg_error($row[1]);
        $self->avg_cluster_per_tile($row[2]);
        $self->avg_cluster_per_tile_raw($row[3]);##no critic
      }
      1;
  } or do {
      croak $EVAL_ERROR;
  };

   return;
}
#given id_run, end
#get the average cluster number per tile for the control lanes
sub get_cluster_control_lane{
  my ($self, $id_run, $end,) = @_;
  
  my $query = q{   
         SELECT ROUND(AVG(temp.clusters_raw*temp.perc_pf_clusters/?)) AS clusters_pf,
                ROUND(AVG(temp.clusters_raw)) AS clusters_raw
         FROM  (SELECT DISTINCT position, tile, clusters_raw, perc_pf_clusters, perc_error_rate_pf
                         FROM  lane_qc lqc, run_tile rt
                         WHERE rt.id_run = ?
                         AND   rt.id_run_tile = lqc.id_run_tile
                         AND   lqc.end = ?
                         AND   lqc.perc_align_pf != 'unknown'
                         ORDER BY tile) temp
               };
  eval {
      my $dbh = $self->util->dbh();
     
      my $sth = $dbh->prepare($query);
      
      $sth->execute($PERCENTAGE, $id_run, $end);

      my @row = $sth->fetchrow_array();
      if(@row){
        $self->avg_cluster_per_tile_control($row[0]);
        $self->avg_cluster_per_tile_raw_control($row[1]);
      }
      1;
  } or do {
      croak $EVAL_ERROR;
  };

   return;
}
#get the actual id_run number based on its end
sub get_actual_id_run{
  my ($self, $id_run, $end) = @_;

  my $actual_id_run = $id_run;
  eval{
    if($end != 1){
      my $paired_id_run = $self->id_run_pair($id_run);
      if($paired_id_run){
        $actual_id_run = $paired_id_run;
      }
    }
    1;
  } or do{
    croak $EVAL_ERROR;
  };

  return $actual_id_run;
}
#given id_run, get the id_run for its paired run
sub get_pair_id_run{
    my ($self, $id_run) = @_;
    
    my $query = q{SELECT id_run_pair
                  FROM run_and_pair
                  WHERE id_run = ?         
                 };
    my $pair_id_run;
    eval {
      my $dbh = $self->util->dbh();
      #print $query, "\n";
      my $sth = $dbh->prepare($query);
      
      $sth->execute($id_run);

      my @row = $sth->fetchrow_array();
      if(@row){
        #print $row[0], "\n";          
        $pair_id_run = $row[0];
      }else{
        $pair_id_run = $id_run;
      }
      1;
    } or do {
      croak $EVAL_ERROR;
    };
    return $pair_id_run;
}
#get the runlist which data alread being calculated
sub get_runlist_done{
    my ($self) = @_;
    
    my $runlist;
    
    my $query = q{SELECT id_run
                  FROM run_graph
                  WHERE yield_gb IS NOT NULL
                  AND avg_cluster_per_tile_raw IS NOT NULL;
                 };
    eval {
      my $dbh = $self->util->dbh();
      #print $query, "\n";
      my $sth = $dbh->prepare($query);
      
      $sth->execute();

      while (my @row = $sth->fetchrow_array()){
        $runlist->{$row[0]} = 1;
      }

      1;
    } or do {
      croak $EVAL_ERROR;
    };

  return $runlist;
}
#get the runlist which data need to be calculated
sub get_runlist_todo{
    my ($self) = @_;
    
    my $runlist_todo;

    my $runlist_done = $self->get_runlist_done();
    
    my $query = q{SELECT tempb.id_run, tempb.end
                    FROM (
                           SELECT COUNT(*) AS count, id_run, end
                           FROM (
                                 SELECT DISTINCT position, rt.id_run, rt.end AS end, tile, clusters_raw
                                 FROM run_tile rt, lane_qc lqc,
                                  (SELECT MAX(tile) AS tile_max, id_run
                                   FROM run_tile
                                   GROUP BY id_run
                                  ) tempa
                                 WHERE rt.id_run_tile = lqc.id_run_tile
                                 AND rt.id_run = tempa.id_run
                                 AND tile = tempa.tile_max
                           ) temp
                          WHERE temp.tile in (100,200,300,330)
                          GROUP BY id_run, end
                          HAVING count = 8
                  )tempb
                 };

    eval {
      my $dbh = $self->util->dbh();
      #print $query, "\n";
      my $sth = $dbh->prepare($query);
      
      $sth->execute();

      while (my @row = $sth->fetchrow_array()){
         my $id_run = $row[0];
         my $end = $row[1];

         #print "$id_run $end\n";
         if(($end == 1) && (!$runlist_done->{$id_run})){
           push @{$runlist_todo}, \@row;
           #print "$id_run $end todo\n";
         }elsif($end == 2){
           my $pair_id_run = $self->get_pair_id_run($id_run);
           if(!$runlist_done->{$pair_id_run}){
             push @{$runlist_todo}, \@row;
             #print "$id_run $end todo\n";
           } #else{
               #print "$id_run $end done\n";
            #}
         } #else{
           #print "$id_run $end done\n";
         #}
      }

      1;
    } or do {
      croak $EVAL_ERROR;
    };

  return $runlist_todo;
}
#given an id_run, get the cycle_count for this run via cache table
sub get_cycle_count_from_cache{
    my ($self, $id_run) = @_;

    my $util = $self->{util};

    my $cycle_count;

    my $cache_cycle_count = npg_qc::model::cache_query->new({
                                 util => $util,
                                 id_run => $id_run,
                                 end =>1,
                                 type =>'cycle_count',
                            });
    my $rows_ref = $cache_cycle_count->get_cache_by_id_type_end();
    if($rows_ref){
      $cycle_count = $rows_ref->[0][0];
    }
    return $cycle_count;
}
#given an id_run, get the cycle_count for this run via direct query
sub get_cycle_count_direct{
    my ($self, $id_run) = @_;

    my $query = q{SELECT MAX(cycle) FROM errors_by_cycle ec, run_tile rt}
             .q{ WHERE rt.id_run = ? AND rt.id_run_tile = ec.id_run_tile};
    my $cycle_count;
    eval {
      my $dbh = $self->util->dbh();
      #print $query, "\n";
      my $sth = $dbh->prepare($query);
      
      $sth->execute($id_run);

      my @row = $sth->fetchrow_array();
      if(@row){
        $cycle_count = $row[0];
      }
      1;
    } or do {
      croak $EVAL_ERROR;
    };
    return $cycle_count;
}
#given an id_run, get the cycle_count for this run
sub get_cycle_count{
  my ($self, $id_run) = @_;

  my $cycle_count_cached = $self->get_cycle_count_from_cache($id_run);
  if($cycle_count_cached){
    
    return $cycle_count_cached;
  }else{
    my $cycle_count_direct = $self->get_cycle_count_direct($id_run);
    if($cycle_count_direct){
      return $cycle_count_direct;
    }
    croak "can not get cycle count\n";
  }
  return;
}

sub calculate_one_run{
  my ($self, $run_id, $end) = @_;

  my $util = $self->{util};
  my $run_graph_model;
  eval{
    my $cycle_count = $self->get_cycle_count($run_id);
    my $actual_id_run = $self->get_actual_id_run($run_id, $end);

    $run_graph_model = npg_qc::model::run_graph->new({
                                  util => $util,
                                  id_run => $actual_id_run,
                               });

    $run_graph_model->get_total_yield_error_cluster($run_id, $end, $cycle_count);
    $run_graph_model->get_cluster_control_lane($run_id, $end);
    if(defined $run_graph_model->yield_gb()
      && defined $run_graph_model->avg_cluster_per_tile_control()){
       $run_graph_model->save();
    }else{
      carp "there is no run graph data for caching for $actual_id_run";
    }

    1;
  } or do{
    croak $EVAL_ERROR;
  };
 
  return $run_graph_model;
}
#get list of runs which data need to be calculated,
#calculate and save to the database
sub calculate_all{
  my ($self) = @_;

  my $util = $self->{util};
  
  my $runlist_todo = $self->get_runlist_todo();

  if (!$runlist_todo) {
    carp 'there are no runs to calculate run_graph';
    return 0;
  }
  carp 'there are '.(scalar @{$runlist_todo}).' to cache run_graph data';

  $util->transactions(0);
  eval{

    foreach my $run_end (@{$runlist_todo}){

      my $run_id = $run_end->[0];
      my $end = $run_end->[1];
      my $run_graph_model;
      eval{
	     my $cycle_count= $self->get_cycle_count($run_id);

	     my $actual_id_run = $self->get_actual_id_run($run_id, $end);
	     #print "$run_id, $end, $cycle_count\n";
	     $run_graph_model = npg_qc::model::run_graph->new({
				                            util => $util,
				                            id_run => $actual_id_run,
			                     });
	     $run_graph_model->get_total_yield_error_cluster($run_id, $end, $cycle_count);
	     $run_graph_model->get_cluster_control_lane($run_id, $end);

	
	     if(defined $run_graph_model->yield_gb()
                  && defined $run_graph_model->avg_cluster_per_tile_control()){
            $run_graph_model->save();
            #carp "run graph data cached for $actual_id_run";
        }else{
            carp "there is no run graph data for caching for $actual_id_run";
        }
	     1;
      } or do{
	     next;
      };
    }
    $util->dbh->commit();
    $util->transactions(1);
    1;
  } or do {
    $util->transactions(1);
    $util->dbh->rollback();
    croak $EVAL_ERROR;
  };
    
  return scalar @{$runlist_todo};
}

sub get_yield_by_run {
  my ($self, $num_runs) = @_;
  
  my @rows;

  eval {
    my $dbh = $self->util->dbh();
    my $query = q{SELECT temp.id_run, temp.yield_gb
                  FROM
                    (SELECT id_run, yield_gb  FROM run_graph ORDER BY id_run DESC LIMIT 0, };
        $query .= $num_runs;
        $query .= q{ )
                  temp
                  ORDER BY id_run ASC
                 };

    my $sth = $dbh->prepare($query);

    $sth->execute();

    while (my @row = $sth->fetchrow_array()) {
      push @rows, \@row;
    }
    1;
  } or do {
    croak $EVAL_ERROR;
  };

  if (!scalar@rows) {
    return [];
  }
  
  return \@rows;
}

sub get_avg_error_by_run {
  my ($self, $num_runs) = @_;
  
  my @rows;

  eval {
    my $dbh = $self->util->dbh();
    my $query = q{SELECT temp.id_run, temp.avg_error
                  FROM
                    (SELECT id_run, avg_error  FROM run_graph ORDER BY id_run DESC LIMIT 0, };
        $query .= $num_runs;
        $query .= q{ )
                  temp
                  ORDER BY id_run ASC
                 };

    my $sth = $dbh->prepare($query);

    $sth->execute();

    while (my @row = $sth->fetchrow_array()) {
      push @rows, \@row;
    }
    1;
  } or do {
    croak $EVAL_ERROR;
  };

  if (!scalar@rows) {
    return [];
  }
  
  return \@rows;
}

sub get_cluster_per_tile_by_run {
  my ($self, $num_runs) = @_;
  
  my @rows;

  eval {
    my $dbh = $self->util->dbh();
    my $query = q{SELECT temp.id_run, temp.avg_cluster_per_tile, temp.avg_cluster_per_tile_raw
                  FROM
                    (SELECT id_run, avg_cluster_per_tile, avg_cluster_per_tile_raw
                     FROM run_graph ORDER BY id_run DESC LIMIT 0, };
        $query .= $num_runs;
        $query .= q{ )
                  temp
                  ORDER BY id_run ASC
                 };

    my $sth = $dbh->prepare($query);

    $sth->execute();

    while (my @row = $sth->fetchrow_array()) {
      push @rows, \@row;
    }
    1;
  } or do {
    croak $EVAL_ERROR;
  };

  if (!scalar@rows) {
    return [];
  }
  
  return \@rows;
}

sub get_cluster_per_tile_control_by_run {
  my ($self, $num_runs) = @_;
  
  my @rows;

  eval {
    my $dbh = $self->util->dbh();
    my $query = q{SELECT temp.id_run, temp.avg_cluster_per_tile_control, temp.avg_cluster_per_tile_raw_control
                  FROM
                    (SELECT id_run, avg_cluster_per_tile_control, avg_cluster_per_tile_raw_control
                     FROM run_graph ORDER BY id_run DESC LIMIT 0, };
        $query .= $num_runs;
        $query .= q{ )
                  temp
                  ORDER BY id_run ASC
                 };

    my $sth = $dbh->prepare($query);

    $sth->execute();

    while (my @row = $sth->fetchrow_array()) {
      push @rows, \@row;
    }
    1;
  } or do {
    croak $EVAL_ERROR;
  };

  if (!scalar@rows) {
    return [];
  }
  
  return \@rows;
}

sub instrument_statistics {
  my ($self) = @_;
  return npg_qc::model::instrument_statistics->new({ util => $self->util() });
}

1;
__END__
=head1 NAME

npg_qc::model::run_graph

=head1 VERSION

$LastChangedRevision: 3347 $

=head1 SYNOPSIS

  my $oRun_graph = npg_qc::model::run_graph->new({util => $util});

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2  fields - a list of fields

=head2  calculate_all - get list of runs which data need to be calculated, calculate and save to the database

=head2  calculate_one_run - calculate and save data for one run

=head2  get_cluster_control_lane - given id_run and end, get the average cluster number per tile for the control lanes

=head2  get_cycle_count - given an id_run, get the cycle_count for this run

=head2  get_cycle_count_direct - given an id_run, get the cycle_count for this run via direct query

=head2  get_cycle_count_from_cache - given an id_run, get the cycle_count for this run via cache table

=head2  get_pair_id_run - given id_run, get the id_run for its paired run

=head2  get_runlist_done - get the runlist which data alread being calculated

=head2  get_runlist_todo - get the runlist which data need to be calculated

=head2  get_total_yield_error_cluster - given id_run, end and the cycle count of one run, get total yield, avg error and avg cluster number per tile for this run

=head2  init - given id_run, find primary key if there is one

=head2  get_actual_id_run - get the actual id_run number based on its end

=head2  get_yield_by_run - given the number for the last runs, get data by run from database

=head2  get_avg_error_by_run - given the number for the last runs, get data by run from database

=head2  get_cluster_per_tile_by_run - given the number for the last runs, get data by run from database

=head2  get_cluster_per_tile_control_by_run - given the number for the last runs, get data by run from database

=head2 instrument_statistics - returns an instrument_statistics object

  my $oInstrumentStatistics = $oRunGraph->instrument_statistics();

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
npg_qc::model
Carp
Readonly

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Guoying Qi

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
