#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-10-15
# Last Modified: $Date: 2008-10-30 12:47:27 +0000 (Thu, 30 Oct 2008) $
# Id:            $Id: 20-view-run_tile.t 3347 2008-10-30 12:47:27Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-3.0/t/20-view-run_tile.t $
#

use strict;
use warnings;
use Test::More  tests => 49 ;# 
use English qw(-no_match_vars);
use SangerPaths qw(core tracking badger ensembl);
use IO::Scalar;
use lib qw{t};
use t::util;
use npg_qc::model::run_tile;
use JSON qw(from_json);

our $VERSION = do { my ($r) = q$LastChangedRevision: 3347 $ =~ /(\d+)/mx; $r; };

use_ok('npg_qc::view::run_tile');
my $util = t::util->new({fixtures =>1});
{
  my $model = npg_qc::model::run_tile->new({util => $util});

  isa_ok($model, 'npg_qc::model::run_tile', '$model');

  my $view  = npg_qc::view::run_tile->new({
    util   => $util,
    model  => $model,
    aspect => 'list_json',
    action => 'list',
  });
  isa_ok($view, 'npg_qc::view::run_tile', '$view');
  is($view->decor(), 0, '$view->decor is 0');
  my $render;
  eval {
    $render = $view->render();
  };
  is($EVAL_ERROR, q{}, 'no croak on render');
  my $json;
  eval { $json = from_json($render); };
  is($EVAL_ERROR, q{}, 'JSON string parsed ok');
  is(scalar keys%{$json}, 1, '1 key as expecting error');
  is($json->{error}, 'no id_run provided', 'correct error message - no id_run provided');
}
{
  my $model = npg_qc::model::run_tile->new({util => $util});

  my $cgi = $util->cgi();
  $cgi->param('id_run', 10);

  my $view  = npg_qc::view::run_tile->new({
    util   => $util,
    model  => $model,
    aspect => 'list_json',
    action => 'list',
  });

  my $render;
  eval {
    $render = $view->render();
  };
  is($EVAL_ERROR, q{}, 'no croak on render');
  my $json;
  eval { $json = from_json($render); };
  is($EVAL_ERROR, q{}, 'JSON string parsed ok');
  is(scalar keys%{$json}, 3, '3 keys expected');

  is($json->{id_run}, 10, 'correct id_run');
  is($json->{id_run_pair}, 11, 'correct id_run_pair');
  isa_ok($json->{run_tiles}, 'ARRAY', 'run_tiles key');
  is(scalar@{$json->{run_tiles}}, 9, 'correct number of run_tiles');
  isa_ok($json->{run_tiles}->[0], 'HASH', '$json->{run_tiles}->[0]');
  is(scalar keys%{$json->{run_tiles}->[0]}, 8, 'correct number of keys');
  is($json->{run_tiles}->[0]->{id_run_tile}, 12, '$json->{run_tiles}->[0]->{id_run_tile} is correct');
  is($json->{run_tiles}->[0]->{id_run}, 10, '$json->{run_tiles}->[0]->{id_run} is correct');
  is($json->{run_tiles}->[0]->{position}, 1, '$json->{run_tiles}->[0]->{position} is correct');
  is($json->{run_tiles}->[0]->{tile}, 330, '$json->{run_tiles}->[0]->{tile} is correct');
  is($json->{run_tiles}->[0]->{end}, 1, '$json->{run_tiles}->[0]->{end} is correct');
  is($json->{run_tiles}->[0]->{row}, 2, '$json->{run_tiles}->[0]->{row} is correct');
  is($json->{run_tiles}->[0]->{col}, 109, '$json->{run_tiles}->[0]->{col} is correct');
  is($json->{run_tiles}->[0]->{avg_newz}, q{}, '$json->{run_tiles}->[0]->{avg_newz} is correct');
}
{
  my $cgi = CGI->new();
  $util->{cgi} = $cgi;

  my $model = npg_qc::model::run_tile->new({util => $util});



  my $view  = npg_qc::view::run_tile->new({
    util   => $util,
    model  => $model,
    aspect => 'read_json',
    action => 'read',
  });

  my $render;
  eval {
    $render = $view->render();
  };
  is($EVAL_ERROR, q{}, 'no croak on render');
  my $json;
  eval { $json = from_json($render); };
  is($EVAL_ERROR, q{}, 'JSON string parsed ok');
  is(scalar keys%{$json}, 1, '1 key as expecting error');
  is($json->{error}, 'no id_run_tile provided', 'correct error message - no id_run_tile provided');
  
}
{
  my $model = npg_qc::model::run_tile->new({util => $util, id_run_tile => 10});

  my $view  = npg_qc::view::run_tile->new({
    util   => $util,
    model  => $model,
    aspect => 'read_json',
    action => 'read',
  });

  my $render;
  eval {
    $render = $view->render();
  };
  is($EVAL_ERROR, q{}, 'no croak on render');
  my $json;
  eval { $json = from_json($render); };
  is($EVAL_ERROR, q{}, 'JSON string parsed ok');

  is(scalar keys%{$json}, 1, 'USE OF ID_RUN_TILE: 1 key expected - single ended run');
  isa_ok($json->{read_one}, 'HASH', 'read_one');
  isa_ok($json->{read_one}->{position_info}, 'HASH', 'read_one->{position_info}');
  isa_ok($json->{read_one}->{tile_qc_data}, 'HASH', 'read_one->{tile_qc_data}');
  isa_ok($json->{read_one}->{errors_by_cycle}, 'ARRAY', 'read_one->{errors_by_cycle}');
  is($json->{read_one}->{position_info}->{id_run}, 3, 'correct id_run for read_one tile');
  is(scalar keys%{$json->{read_one}->{position_info}}, 7, '7 keys expected for position_info');
  is(scalar keys%{$json->{read_one}->{tile_qc_data}}, 8, '8 keys expected for tile_qc_data');
}
{
  my $model = npg_qc::model::run_tile->new({util => $util});
  my $cgi = $util->cgi();
  $cgi->param('id_run', 3);
  $cgi->param('position', 2);
  $cgi->param('tile', 7);

  my $view  = npg_qc::view::run_tile->new({
    util   => $util,
    model  => $model,
    aspect => 'read_json',
    action => 'read',
  });

  my $render;
  eval {
    $render = $view->render();
  };
  is($EVAL_ERROR, q{}, 'no croak on render');
  my $json;
  eval { $json = from_json($render); };
  is($EVAL_ERROR, q{}, 'JSON string parsed ok');

  is(scalar keys%{$json}, 1, 'USE OF ID_RUN, POSITION, TILE: 1 key expected - single ended run');
  isa_ok($json->{read_one}, 'HASH', 'read_one');
  isa_ok($json->{read_one}->{position_info}, 'HASH', 'read_one->{position_info}');
  isa_ok($json->{read_one}->{tile_qc_data}, 'HASH', 'read_one->{tile_qc_data}');
  isa_ok($json->{read_one}->{errors_by_cycle}, 'ARRAY', 'read_one->{errors_by_cycle}');
  is($json->{read_one}->{position_info}->{id_run}, 3, 'correct id_run for read_one tile');
  is(scalar keys%{$json->{read_one}->{position_info}}, 7, '7 keys expected for position_info');
  is(scalar keys%{$json->{read_one}->{tile_qc_data}}, 8, '8 keys expected for tile_qc_data');


}
