#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-07-22
# Last Modified: $Date: 2008-10-30 12:47:27 +0000 (Thu, 30 Oct 2008) $
# Id:            $Id: 20-view-summary.t 3347 2008-10-30 12:47:27Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-3.0/t/20-view-summary.t $
#

use strict;
use warnings;
use Test::More tests => 19;
use English qw(-no_match_vars);
use SangerPaths qw(core tracking badger ensembl);
use IO::Scalar;
use lib qw{t};
use t::util;
use Carp;
use npg_qc::model::summary;
use JSON qw(from_json);

our $VERSION = do { my ($r) = q$LastChangedRevision: 3347 $ =~ /(\d+)/mx; $r; };

use_ok('npg_qc::view::summary');

my $util  = t::util->new({fixtures => 1});
{
  my $cgi = $util->cgi();
  $cgi->param('id_run', 1);

  my $model = npg_qc::model::summary->new({util => $util});

  my $view  = npg_qc::view::summary->new({
    util   => $util,
    model  => $model,
    action => 'add',
    aspect => 'add_ajax',
  });
  isa_ok($view, 'npg_qc::view::summary', '$view');

  my $render;
  eval { $render = $view->render(); };
  is($EVAL_ERROR, q{}, 'no croak on render add_ajax');

  ok($util->test_rendered($render, 't/data/rendered/html/summary_add_ajax.html'), 'add_ajax rendered ok');
}
{
  my $model = npg_qc::model::summary->new({util => $util});

  my $view  = npg_qc::view::summary->new({
    util   => $util,
    model  => $model,
    action => 'list',
    aspect => 'list_xml',
  });

  my $render;
  eval { $render = $view->render(); };
  is($EVAL_ERROR, q{}, 'no croak on render list_xml');
  ok($util->test_rendered($render, 't/data/rendered/summary_list_xml.xml'), 'list_xml rendered ok');
}
{
  my $model = npg_qc::model::summary->new({util => $util, id_run => 3});

  my $view  = npg_qc::view::summary->new({
    util   => $util,
    model  => $model,
    action => 'read',
    aspect => 'read',
  });

  my $render;
  eval { $render = $view->render(); };
  is($EVAL_ERROR, q{}, 'no croak on render read');
  ok($util->test_rendered($render, 't/data/rendered/html/summary_read.html'), 'read rendered ok');

}
{
  my $model = npg_qc::model::summary->new({util => $util, id_run => 3});

  my $view  = npg_qc::view::summary->new({
    util   => $util,
    model  => $model,
    action => 'read',
    aspect => 'read_json',
  });

  my $render;
  eval { $render = $view->render(); };
  is($EVAL_ERROR, q{}, 'no croak on render read');
  my $json;
  eval { $json = from_json($render); };
  is($EVAL_ERROR, q{}, 'JSON string parsed ok');
  is(scalar keys%{$json}, 3, '3 keys as single end run');
  isa_ok($json->{chip_summary}, 'HASH', '$json->{chip_summary}');
  is(scalar keys%{$json->{chip_summary}}, 14, '14 keys for chip_summary');
  isa_ok($json->{lane_results_summary_read_one}, 'HASH', '$json->{lane_results_summary_read_one}');
  is(scalar keys%{$json->{lane_results_summary_read_one}}, 2, '2 lanes for lane_results_summary_read_one');
  isa_ok($json->{lane_results_summary_read_one}->{1}, 'HASH', '$json->{lane_results_summary_read_one}->{1}');
  isa_ok($json->{lane_results_summary_read_one}->{2}, 'HASH', '$json->{lane_results_summary_read_one}->{2}');
  is(scalar keys%{$json->{lane_results_summary_read_one}->{1}}, 17, '17 keys for lane 1');
  is($json->{id_run}, 3, 'correct id_run');

}
